<?php

namespace Makeaholic\Woo;

defined( 'ABSPATH' ) || exit;

class Discount {

	const RECOMMEND_PLUGIN_VERSION = '2.28.2';

	protected static $instance = null;

	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function initialize() {
		if ( ! $this->is_activate() ) {
			return;
		}

		// Check old version installed.
		if ( defined( 'WAD_VERSION' ) && version_compare( WAD_VERSION, self::RECOMMEND_PLUGIN_VERSION, '<' ) ) {
			add_action( 'admin_notices', [ $this, 'admin_notice_minimum_plugin_version' ] );
		}

		/**
		 * Fix quantity discount not applied on ajax request.
		 */
		add_filter( 'woocommerce_cart_subtotal', [ $this, 'get_cart_subtotal' ], 100, 1 );

		/**
		 * Remove quantity based pricing table from plugin then add custom HTML
		 *
		 * @see WAD_Discount::get_quantity_pricing_tables()
		 * @see add_quantity_pricing_tables_html()
		 */
		add_filter( 'wad_get_quantity_pricing_tables', '__return_empty_string', 99, 3 );
		add_action( 'woocommerce_before_add_to_cart_button', [ $this, 'add_quantity_pricing_tables_html' ] );
	}

	public function is_activate() {
		if ( defined( 'WAD_VERSION' ) ) {
			return true;
		}

		return false;
	}

	public function get_version() {
		return WAD_VERSION;
	}

	public function admin_notice_minimum_plugin_version() {
		makeaholic_notice_required_plugin_version( 'Woo Advanced Discounts', self::RECOMMEND_PLUGIN_VERSION );
	}

	/**
	 * Remove is_cart & is_checkout conditions to working properly in AJAX
	 *
	 * @param $subtotal
	 *
	 * @return string
	 * @see \WAD_Discount::get_cart_subtotal()
	 *
	 */
	public function get_cart_subtotal( $subtotal ) {
		$new_subtotal = 0;
		$items        = WC()->cart->get_cart_contents();

		foreach ( $items as $item => $values ) {
			/**
			 * @var \WC_Product $product_obj
			 */
			$product_obj  = $values['data'];
			$price        = $product_obj->get_price();
			$quantity     = $values['quantity'];
			$new_subtotal += $price * $quantity;
		}
		$subtotal = wc_price( $new_subtotal );

		return $subtotal;
	}

	public function add_quantity_pricing_tables_html() {
		/**
		 * @var \WC_Product|\WC_Product_Variable $product
		 */
		global $product;
		$product_id       = $product->get_id();
		$quantity_pricing = get_post_meta( $product_id, 'o-discount', true );

		if ( version_compare( $this->get_version(), '2.28.2', '>' ) ) { // After ver 2.28.2
			$rules_type = \O_Utils::get_proper_value( $quantity_pricing, 'rules-type', 'intervals' );
		} else {
			$rules_type = get_proper_value( $quantity_pricing, 'rules-type', 'intervals' );
		}

		if ( ! isset( $quantity_pricing['enable'] ) ) {
			return;
		}

		if ( ! isset( $quantity_pricing['rules'] ) && ! isset( $quantity_pricing['rules-by-step'] ) ) {
			return;
		}
		?>
		<?php
		if ( 'intervals' === $rules_type ) {
			if ( $product->get_type() === 'variable' ) {
				$available_variations = $product->get_available_variations();
				foreach ( $available_variations as $variation ) {
					$product_price = $variation['display_price'];
					$this->get_quantity_pricing_table( $variation['variation_id'], $quantity_pricing, $product_price );
				}
			} else {
				$product_price = $product->get_price();
				$this->get_quantity_pricing_table( $product_id, $quantity_pricing, $product_price, true );
			}
		} else if ( 'steps' === $rules_type ) {

			if ( $product->get_type() === 'variable' ) {
				$available_variations = $product->get_available_variations();
				foreach ( $available_variations as $variation ) {
					$product_price = $variation['display_price'];
					$this->get_steps_quantity_pricing_table( $variation['variation_id'], $quantity_pricing, $product_price );
				}
			} else {
				$product_price = $product->get_price();
				$this->get_steps_quantity_pricing_table( $product_id, $quantity_pricing, $product_price, true );
			}
		}
	}

	private function get_steps_quantity_pricing_table( $product_id, $quantity_pricing, $product_price, $display = false ) {
		global $product;
		$classes = 'quantity-discount-table wad-qty-pricing-table';
		?>
		<table class="<?php echo esc_attr( $classes ) ?>" data-id="<?php echo esc_attr( $product_id ); ?>"
			<?php if ( ! $display ) : ?>
				style="display: none;"
			<?php endif; ?>
		>
			<thead>
			<tr>
				<th class="col-quantity-discount-name">&nbsp;</th>
				<th class="col-quantity-regular-price">&nbsp;</th>
				<th class="col-quantity-discount-price">&nbsp;</th>
				<th class="col-quantity-discount-badge">&nbsp;</th>
			</tr>
			</thead>
			<tbody>
			<?php
			$discount_price = $product_price;
			$discount_type  = $quantity_pricing['type'];
			foreach ( $quantity_pricing['rules-by-step'] as $rule ) :
				if ( '' === $rule['every'] ) {
					continue;
				}

				$input_disable = false;
				$min_quantity  = apply_filters( 'woocommerce_quantity_input_min', $product->get_min_purchase_quantity(), $product );
				$max_quantity  = apply_filters( 'woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product );
				$quantity      = intval( $rule['every'] );

				if ( $quantity < $min_quantity || ( $quantity > $max_quantity && $max_quantity !== - 1 ) ) {
					$input_disable = true;
				}

				$discount_value_text = $rule['discount'];

				switch ( $discount_type ) {
					case 'fixed':
						$discount_price = $product_price - $rule['discount'];

						$discount_percentage = round( ( $product_price - $discount_price ) / $product_price * 100 );

						$discount_value_text = $discount_percentage . '%';
						break;
					case 'percentage':
						$discount_price = $product_price - ( $product_price * $rule['discount'] ) / 100;

						$discount_value_text = $rule['discount'] . '%';
						break;
				}

				$quantity_regular_price = $product_price * $quantity;
				$quantity_sale_price    = $discount_price * $quantity;

				$row_classes = 'quantity-discount-item';

				if ( $input_disable ) {
					$row_classes .= ' disabled';
				}
				?>
				<tr class="<?php echo esc_attr( $row_classes ); ?>">
					<td class="col-quantity-discount-name">
						<label class="quantity-discount-name">
							<input type="radio" name="quantity_discount"
							       value="<?php echo esc_attr( $quantity ); ?>" <?php if ( $input_disable ) {
								echo 'disabled';
							} ?> />
							<?php echo sprintf( esc_html__( 'Buy %s :', 'makeaholic' ), $quantity ) ?>
						</label>
					</td>
					<td class="col-quantity-regular-price">
						<div class="price quantity-regular-price">
							<del><?php echo wc_price( $quantity_regular_price ); ?></del>
						</div>
					</td>
					<td class="col-quantity-discount-price">
						<div class="price quantity-discount-price">
							<ins><?php echo wc_price( $quantity_sale_price ); ?></ins>
						</div>
					</td>
					<td class="col-quantity-discount-badge">
						<div class="quantity-discount-badge">
							<?php echo sprintf( esc_html__( 'Save %s', 'makeaholic' ), $discount_value_text ); ?>
						</div>
					</td>
				</tr>
			<?php endforeach; ?>
			</tbody>
		</table>
		<?php
	}

	private function get_quantity_pricing_table( $product_id, $quantity_pricing, $product_price, $display = false ) {
		global $product;
		$classes = 'quantity-discount-table wad-qty-pricing-table';
		?>
		<table class="<?php echo esc_attr( $classes ) ?>" data-id="<?php echo esc_attr( $product_id ); ?>"
			<?php if ( ! $display ) : ?>
				style="display: none;"
			<?php endif; ?>
		>
			<thead>
			<tr>
				<th class="col-quantity-discount-name">&nbsp;</th>
				<th class="col-quantity-regular-price">&nbsp;</th>
				<th class="col-quantity-discount-price">&nbsp;</th>
				<th class="col-quantity-discount-badge">&nbsp;</th>
			</tr>
			</thead>
			<tbody>
			<?php
			$discount_price    = $product_price;
			$discount_type     = $quantity_pricing['type'];
			foreach ( $quantity_pricing['rules'] as $rule ) :
				$discount_label = '';
				$min_quantity  = apply_filters( 'woocommerce_quantity_input_min', $product->get_min_purchase_quantity(), $product );
				$max_quantity  = apply_filters( 'woocommerce_quantity_input_max', $product->get_max_purchase_quantity(), $product );
				$input_disable = false;

				if ( '' !== $rule['min'] ) {
					$quantity = $rule['min'];
				} elseif ( '' !== $rule['max'] ) {
					$quantity = $rule['max'];
				}

				if ( empty( $quantity ) ) {
					continue;
				}

				if ( $quantity < $min_quantity || ( $quantity > $max_quantity && $max_quantity !== - 1 ) ) {
					$input_disable = true;
				}

				if ( '' !== $rule['min'] && '' !== $rule['max'] ) {
					if ( $rule['min'] !== $rule['max'] ) {
						$discount_label = sprintf( esc_html__( 'Buy %1$s to %2$s:', 'makeaholic' ), $rule['min'], $rule['max'] );
					} else {
						$discount_label = sprintf( esc_html__( 'Buy %s :', 'makeaholic' ), $quantity );
					}
				} elseif ( '' !== $rule['min'] ) {
					$discount_label = sprintf( esc_html__( 'Buy from %s :', 'makeaholic' ), $quantity );
				} elseif ( '' !== $rule['max'] ) {
					$discount_label = sprintf( esc_html__( 'Buy %s :', 'makeaholic' ), $quantity );
				}

				$discount_value_text = $rule['discount'];

				switch ( $discount_type ) {
					case 'fixed':
						$discount_price = $product_price - $rule['discount'];

						$discount_percentage = round( ( $product_price - $discount_price ) / $product_price * 100 );
						$discount_value_text = $discount_percentage . '%';
						break;
					case 'percentage':
						$discount_price = $product_price - ( $product_price * $rule['discount'] ) / 100;

						$discount_value_text = $rule['discount'] . '%';
						break;
				}

				$quantity               = intval( $quantity );
				$quantity_regular_price = $product_price * $quantity;
				$quantity_sale_price    = $discount_price * $quantity;

				$row_classes = 'quantity-discount-item';

				if ( $input_disable ) {
					$row_classes .= ' disabled';
				}
				?>
				<tr class="<?php echo esc_attr( $row_classes ); ?>">
					<td class="col-quantity-discount-name">
						<label class="quantity-discount-name">
							<input type="radio" name="quantity_discount"
							       value="<?php echo esc_attr( $quantity ); ?>" <?php if ( $input_disable ) {
								echo 'disabled';
							} ?> />
							<?php echo esc_html( $discount_label ); ?>
						</label>
					</td>
					<td class="col-quantity-regular-price">
						<div class="price quantity-regular-price">
							<del><?php echo wc_price( $quantity_regular_price ); ?></del>
						</div>
					</td>
					<td class="col-quantity-discount-price">
						<div class="price quantity-discount-price">
							<ins><?php echo wc_price( $quantity_sale_price ); ?></ins>
						</div>
					</td>
					<td class="col-quantity-discount-badge">
						<div class="quantity-discount-badge">
							<?php echo sprintf( esc_html__( 'Save %s', 'makeaholic' ), $discount_value_text ); ?>
						</div>
					</td>
				</tr>
			<?php endforeach; ?>
			</tbody>
		</table>
		<?php
	}
}

Discount::instance()->initialize();
